#!/usr/bin/python3

import scvelo as scv
import os, sys
import anndata
import pandas as pd
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import numpy as np



pc_info = {}
#data_list = open(sys.argv[1], 'r')
data_list = open('celltype_list_for_scvelo', 'r')
for line in data_list:
        line = line.strip()
        cell, pc = line.split("\t")
        pc_info[cell] = int(pc)

data_list.close()

print("reading loom...\n")
#total_data = anndata.read_loom(sys.argv[2])
total_data = anndata.read_loom('merge_tcell_filter.loom')
print("embedded\n")

for cell_name in pc_info:
        print(cell_name + "\n")
        umap_cord = pd.read_csv(cell_name + "_embeddings.csv")
        color_clusters = pd.read_csv(cell_name + "_cluster_color.csv")
        cell_clusters = pd.read_csv(cell_name + "_clusters.csv")
        har_cord = pd.read_csv(cell_name + "_harmony.csv")
        var = pd.read_csv(cell_name + "_var.csv")
        mat = pd.read_csv(cell_name + "_mat.csv")
        sample_obs = pd.read_csv(cell_name + "_loom_id.csv")
        data = total_data[np.isin(total_data.obs.index,sample_obs["x"])]
        data.var_names_make_unique()
        data = data[:,np.isin(data.var_names, var.x)]
        data.var_names_make_unique()
        data = data[:,np.isin(data.var_names, var.x)]
        temp1 = [list(var.x).index(val) for val in list(data.var_names) if val in set(var.x)]
        temp2 = [list(sample_obs.x).index(val) for val in list(data.obs_names) if val in set(sample_obs.x)]
        mat1 = mat.iloc[temp2, temp1]
        data.X = mat1
        umap_cord1 = umap_cord.iloc[temp2,]
        har_cord1 = har_cord.iloc[temp2,]
        cell_clusters1 = cell_clusters.iloc[temp2,]
        color_clusters1 = color_clusters.iloc[temp2,]
        data.obs['clusters'] = list(cell_clusters1.x)
        scv.pp.normalize_per_cell(data)
        scv.pp.log1p(data)
        data.obsm['X_umap'] = umap_cord1.iloc[:,1:].values
        data.uns['Cluster_colors'] = color_clusters1
        scv.pp.pca(data, n_comps=pc_info[cell_name])
        data.obsm['X_pca'] = np.array(har_cord1)
        scv.pp.moments(data, n_pcs=pc_info[cell_name], n_neighbors=30)
        scv.tl.velocity(data, mode='stochastic')
        scv.tl.velocity_graph(data, basis = "umap")
        scv.pl.velocity_embedding_grid(data, basis="umap", size = 15, alpha = 1,arrow_length=3, arrow_size=1.5, dpi=2000, color = data.uns['Cluster_colors'].x, xlim = (-15,20), ylim = (-15,20))
        plt.savefig(cell_name + "_grid.png")
        plt.close()
        scv.pl.velocity_embedding_stream(data, basis="umap", size = 15, alpha = 1,  dpi=2000, color = data.uns['Cluster_colors'].x, xlim = (-15,20), ylim = (-15,20))
        plt.savefig(cell_name + "_stream.png")
        plt.close()
        #paga
        #scv.utils.get_transition_matrix(data).toarray()
        data.uns['neighbors']['distances'] = data.obsp['distances']
        data.uns['neighbors']['connectivities'] = data.obsp['connectivities']
        scv.tl.paga(data, groups = 'clusters')
        df = scv.get_df(data, 'paga/transitions_confidence', precision=2).T
        df.to_csv(cell_name + "_paga_df", sep = "\t")
        scv.pl.paga(data, basis = 'umap', size = 50, alpha=0.1, min_edge_width = 2, node_size_scale = 1.5)
        plt.savefig(cell_name + "_paga_umap.png")
        plt.close()
        data.write_loom(filename = cell_name + "_scvelo.loom")



